//
// Copyright (c) Microsoft Corporation. All rights reserved.
//

/**
 * @file att_manager_api.h
 *
 * @brief Defines the types and functions used for attestation.
 *
 * @note You MUST NOT use any symbols (macros, functions, structures, enums, etc.)
 * prefixed with an underscore ('_') directly in your application code. These symbols
 * are part of the SDK's internal implementation; we do not document these symbols
 * and they are subject to change in future versions of the SDK which would break your code.
 */

#ifndef _ATT_MANAGER_API_H
#define _ATT_MANAGER_API_H

#include <stdint.h>
#include <stddef.h>

#include "att_result.h"

/**
 * Represents an attestation session handle.
 */
typedef uint64_t att_session_handle;

/**
 * TPM attestation session type.
 *
 * Use \ref att_session_params_tpm structure in the call to \ref att_create_session.
 */
#define ATT_SESSION_TYPE_TPM "TPM"

/**
 * Creates a new attestation session.
 *
 * @param[in] session_type A string that represents the session type to create. The only type currently supported is ATT_SESSION_TYPE_TPM.
 *
 * @param[in] session_params The address of a buffer that contains the session parameters. The type of the session parameters structure is dependent on the session type.
 *
 * @param[out] session A pointer to a variable that receives the handle to the newly created session. The caller MUST free this handle by calling \ref att_close_session.
 *
 * @return An \ref att_result value indicating the result of the operation.
 */
att_result att_create_session(
    const char* session_type,
    const void* session_params,
    att_session_handle* session);

/**
 * Performs attestation. This method should be called in a loop until the \p complete parameter returns true.
 * 
 * @param[in] session The attestation session handle.
 * 
 * @param[in] input The value of this parameter is generally \c NULL on the first call to the method.
 *                  On subsequent calls to the method, the value of this parameter is the buffer returned by the attestation service.
 * 
 * @param[in] input_size The number of bytes in the \p input buffer.
 * 
 * @param[out] output The address of the buffer that receives the payload to be sent to the attestation service. The caller MUST free this buffer by calling \ref att_free_buffer.
 * 
 * @param[out] output_size A pointer to a variable that receives the number of bytes copied to \p output.
 * 
 * @param[out] complete Indicates whether attestation is complete.
 * 
 * @return An \ref att_result value indicating the result of the operation.
 */ 
att_result att_attest(
    att_session_handle session,
    const uint8_t* input,
    size_t input_size,
    uint8_t** output,
    size_t* output_size,
    bool* complete);

/**
 * Gets the attestation report generated by the attestation service. This method must be called after attestation has completed successfully.
 *
 * @param[in] session The attestation session handle.
 *
 * @param[out] report The address of the buffer that receives the report. The caller MUST free this buffer by calling \ref att_free_buffer.
 *
 * @param[out] report_size A pointer to a variable that receives the number of bytes copied to \p report.
 *
 * @return An \ref att_result value indicating the result of the operation.
 */
att_result att_get_report(
    att_session_handle session,
    uint8_t** report,
    size_t* report_size);

/**
 * Closes the attestation session and releases all the underlying resources.
 *
 * @param[in] session The attestation session handle.
 *
 * @return An \ref att_result value indicating the result of the operation.
 */
att_result att_close_session(
    att_session_handle session);

/**
 * Frees a buffer allocated by the SDK.
 *
 * @param[in] buffer The buffer to be freed.
 *
 * @return An \ref att_result value indicating the result of the operation.
 */
att_result att_free_buffer(
    uint8_t* buffer);

#endif  // _ATT_MANAGER_API_H
